package fr.velossity.sample.device.impl;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.Filter;
import org.osgi.framework.ServiceReference;
import org.osgi.util.tracker.ServiceTracker;
import org.osgi.util.tracker.ServiceTrackerCustomizer;

import fr.velossity.sample.device.eventing.MeasurementProducer;

/**
 * Just instantiates the consumer.
 * @author C. Saint-Marcel
 * 
 */
public class Activator implements BundleActivator {
	
	/**
	 * Tracker extension for producers.
	 */
	public class ProducerTracker extends ServiceTracker {

		/**
		 * Default inherited constructor.
		 * @param context the bundle context
		 * @param filter an optional filter
		 * @param customizer an optional customizer
		 */
		public ProducerTracker(BundleContext context, Filter filter,
				ServiceTrackerCustomizer customizer) {
			super(context, filter, customizer);
		}

		/**
		 * @see ServiceTracker#addingService(ServiceReference)
		 */
		public Object addingService(ServiceReference reference) {
			MeasurementProducer producer = (MeasurementProducer) context.getService(reference);
			producer.addListener(myConsumer);
			return producer;
		}

		/**
		 * @see ServiceTracker#removedService(ServiceReference, Object)
		 */
		public void removedService(ServiceReference reference, Object service) {
			MeasurementProducer producer = (MeasurementProducer) service;
			producer.removeListener(myConsumer);
		}		
	}

	/**
	 * The tracker for producers.
	 */
	private ProducerTracker pTracker;

	/**
	 * An internal reference to the POJO consumer.
	 */
	private Consumer myConsumer;

	/**
	 * @see BundleActivator#start(BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		myConsumer = new Consumer("consumer1");
		myConsumer.activate(context);
		
		// DECLARES and ACTIVATE the tracker
		pTracker = new ProducerTracker(context, context.createFilter("(identifier=producer1*)"), null);
		pTracker.open();
	}

	/**
	 * @see BundleActivator#stop(BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		myConsumer.deactivate(context);
		myConsumer = null;
		pTracker.close();
	}
}
